; reaperActions dialog


Include "ReaperGlobals.jsh"
include "reaperActions.jsm"
Import "reaper.jsd"
Import "snow_msaa.jsd"
include "msaa.jsh"
Import "snow_uia.jsd"

globals
	string rprActBrailleLine,
	int rprAFilterPrefixes,
	string rprALastPrefix,
	int rprASelectedItem



Int Function IsActionsDialog (handle hReal)
; yes if focused anywhere in the dialog
var	
	string name,
	object o,
	int id

Let id = 0
Let o = objGetParent (rprFocusObject, id)
Let o = objGetParent (o, id)
Let name = o.accName(0)
if (name == "Actions") then
	return (1)
endif

Let rprActBrailleLine = ""
return (0)
EndFunction


String Function actBrailleLine ()
return (rprActBrailleLine)
EndFunction


Function actToggleVerbosity ()
var
	string s
Let rprASelectedItem = rprASelectedItem + 1
if (rprASelectedItem > 4) then
	Let rprASelectedItem = 1
endif
Let s = StringSegment (msgVerbosityList, "|", rprASelectedItem)
SayString (s)
IniWriteInteger ("options", "actionsVerbosity", rprASelectedItem, "reaper.ini")
EndFunction


Function actVerbosityInit ()
Let rprASelectedItem = IniReadInteger ("options", "actionsVerbosity", 3, "reaper.ini")
Let rprAFilterPrefixes = IniReadInteger ("options", "actionPrefixes", 0, "reaper.ini")
EndFunction


Function actPrefixToggle ()
var
	string s
	
Let rprAFilterPrefixes = rprAFilterPrefixes+1
if (rprAFilterPrefixes > 1) then
	Let rprAFilterPrefixes = 0
endif
IniWriteInteger ("options", "actionPrefixes", rprAFilterPrefixes, "reaper.ini")
Let s = StringSegment (msgPrefixes, "|", rprAFilterPrefixes+1)
SayString (s)
Let rprALastPrefix = ""
EndFunction


String Function actFilterPrefixes (string keyValue)
var
	string s,
	int i,
	String sPrefix
	
if (rprAFilterPrefixes) then
	Let i = StringContains (keyValue, ":")
	if (i > 0) then
		Let sPrefix = StringLeft (keyValue, i-1)
		if (sPrefix == rprALastPrefix) then
			Let keyValue = StringChopLeft (keyValue, (i+1))
		else
			; new prefix
			Let rprALastPrefix = sPrefix
		endif
	else
		; no prefix detected
		Let rprALastPrefix= ""
	endif ; no prefix
else
	; filtering disabled
	Let rprALastPrefix = ""
endif
return  (keyValue)		
EndFunction


Function actSpeakSelectedItem ()
; called for up and down arrow int he actions list
var
	int iSelected,
	int id,
	object o,
	string sState,
	string sOut,
	string sDescription,
	string sHotKey

; about actions dialog down arrow from the filter edit, you land in the list.
; first time we land in here,  nothing has focus,  so establish that
Let o = getObjectFromEvent (hReaperFocus,-4,0,ID)
Let id = o.accFocus()
if (id == 0)  then
	if (o.accChildCount > 1) then
		o.accSelect (msaa_SELFLAG_TAKEFOCUS, 0)
		pause()
		o.accSelect (msaa_SELFLAG_TAKEFOCUS, 1)
	else 
		sayString ("empty list") 
		return 
	endif
endif

Let sHotKey = NameAndKeyFromFocusedObject ("Description", sDescription, sState, iSelected)
Let sDescription = actFilterPrefixes (sDescription)

; compose output
if (rprASelectedItem == 1) then
	if (sState == "blank") then
		Let sOut = sDescription
	else
		; has a state
		Let sOut = FormatString ("%1, %2 %3", sDescription, msgItemState, sState)
	endif
elif (rprASelectedItem == 2) then
	Let sOut = sHotKey
elif (rprASelectedItem == 3) then
	Let sOut = FormatString ("%1: %2",sDescription, sHotKey)
elif (rprASelectedItem == 4) then
	Let sOut = FormatString ("%1: %2",sHotKey, sDescription)
endif
Let rprActBrailleLine = sOut
if (not iSelected) then
	SayString ("not selected")
endif
SayString (sOut)		
EndFunction




Function actSpeakThisItem (string sKeyword, int isDoubleClick)
; called for the function keys, to speak a particular item
; from focused object
var
	object oUIA,
	int iSelected,
	string sName,
	string sPath,
	string sState,
	string keyValue

if (isDoubleClick) then
	Let sPath = GetJAWSSettingsDirectory () + "\\reaperClick2.wav"
	PlaySound (sPath)
	delay(2)
endif

Let sName = NameAndKeyFromFocusedObject	 (sKeyword, keyValue, sState, iSelected)
; for description,  get the unTruncated version
if (sKeyword == "Description") then
	Let oUIA = uGetFocus()
	Let oUIA = uTraverse (oUIA,  "d1|n1")
	Let keyValue = oUIA.name
endif

if (sKeyword == "Hotkey") then
	if (sName) then
		SayString (sName)
	else
		SayString (msgNoHotkey)
	endif
elif (sKeyword == "Description") then
	Let keyValue = actFilterPrefixes (keyValue)
	SayString (keyValue)
	if (sState != "blank") then
		sayString (msgItemState + sState)
	endif
else
	if (sKeyword == "State") then
		if (keyValue != "blank") then
			Let keyValue = msgItemState + keyValue
		endif
	endif
	SayString (keyValue)
	if (isDoubleClick) then
		CopyToClipboard (keyValue)
		SayString ("copied to clipboard")
	endif
endif
EndFunction
		
