; Reaper braille timeline support
; use this under reaper.jss
; provides the ability to use the routing buttons to move around the file.

include "hjGlobal.jsh"
Include "reaperGlobals.jsh"
include "reaperAPI.jsh"
include "reaper.jsm"

Import "reaper.jsd"
import "reaperAPI.jsd"


globals
	int CustomScalingStartBTL, ; integer millisedonds
	int CustomScalingLengthBTL,
	int ScalingBTL  ; 0=full, 1=time selection 2= custom
	


Int Function IsTimelineEnabled ()
return (TimeLineModeRPR & 1)
EndFunction


Function IsTimelineStatic ()
return (TimeLineModeRPR & 2)
EndFunction

Function ScaleToTimeSelectionBTL ()
; should we scale to the time selection,or to the full project
return (ScalingBTL == 1)
EndFunction


Int Function ScaleToCustomBTL ()
return (ScalingBTL == 2)
EndFunction


Function ScaleToProject ()
return (ScalingBTL == 0)
EndFunction



const cn_scalar = 1000
Int Function interpolateBTL (int reference, int signal, int span, int scale)
; returns a fraction of scale,  defined by (signal - reference) / span
; used fixed point arithmetic to avoid integer truncation

return ((((signal - reference) * scale) + (span/2)) / span)
EndFunction



Function CaptureCustomScalingBTL ()
var
	string class,
	int iSelectionStart,
	int iSelectionLength

Let class = getWindowClass (hreaperFocus)
if (IsTrackList (class)) then
	Let iSelectionStart = TimeToMilliseconds (apiGetSelectionStart (tf_minsec))
	Let iSelectionLength = TimeToMilliseconds (apiGetSelectionLength(tf_MINSEC))
	if (iSelectionLength != 0) then
		CustomScalingStartBTL = iSelectionStart
		CustomScalingLengthBTL = iSelectionLength
		SayString (msgCustomScalingComplete)
	else
		SayString (msgNoSelection)
	endif
endif
EndFunction

Int Function BrailleToPlaytime (string sKeyName)
; when a routing button is pressed,  this changes the play cursor time
var
	string sTime,
	int v,
	int r,
	int base,
	int max,
	string class

if (sKeyName == "Braille Routing") then
	; span to full project,  or to time selection
	Let sTime = "0:0.000"
	if (ScaleToTimeSelectionBTL ()) then
		Let sTime = apiGetSelectionLength(tf_MINSEC)
		; that returns 0:0.000 if nothing selected 
	endif 
	if (sTime == "0:0.000") then
		; nothing selected,  or not allowed
		Let sTime = apiGetProjectLength (TF_MINSEC)
		Let base = 0 ; start span at start of project
	else
		Let base = TimeToMilliSeconds (apiGetSelectionStart (tf_minsec))
	endif
	SpeechOff()
	if (ScaleToCustomBTL ()) then
		; use the custom range
		Let v = CustomScalingLengthBTL
		Let base = CustomScalingStartBTL
	else
		; standard scaling
		Let v = TimeToMilliSeconds(sTime)
	endif
	Let max = BrailleGetCellCount ()
	Let r = GetLastBrailleRoutingKey () - 1
	let v = base + ((v*r)/max)
	Let sTime = MillisecondsToTime (v)
	Let TimeLineLastPressTimeRPR = sTime
	{control+j}
	TypeString (sTime)
	EnterKey()
	Pause()
	SpeechOn()
	if (IsPlaying () == 0) then
		SayString (ParseTime (apiGetCurrentPositionTV ()))
	endif
	return (1)
endif
return(0)
EndFunction

const
	cs_cursor = "g"
String Function BuildBrailleLineBTL ()
;called from brailleCallbackObjecIdentify
; make the display showa single symbol position by Cursor time
var
	int iCursor,  ; these three I variables are the number of spaces from left
	int iSelectionStart,
	int iSelectionEnd,
	int iSelectionLength,  ; number of dashes
	
; time values in seconds
	int base,
	int selectionStart,
	int selectionLength,
	int selectionEnd,
	int projectLength,

	int span,
	int r,
	int cellCount,
	string sCursor,
	string spaces80 = "                                                                                ",
	string dashes80 = "--------------------------------------------------------------------------------",
	int showSelectedRegion = 0,
	string sLine

; handle the track ruler first,  in case they are selecting tracks using routing buttons
if (ShowingTrackRulerBTL ()) then
	Let sLine = "abcdefghijklmnopqrstuvxyz"
	return (sLine)
endif

; now, handle braille time line
; scale to the full project?  or to time selection
Let ProjectLength = TimeTomilliSeconds (apiGetProjectLength (tf_minsec))
Let selectionStart = TimeTomilliSeconds (APIGetSelectionStart (tf_minsec))
Let selectionLength = TimeTomilliSeconds ( apiGetSelectionLength (tf_minsec))
Let selectionEnd = selectionStart + selectionLength

; scale to entire project?  Or only to time selection.
Let base = 0
Let span = projectLength
if (selectionLength > 0) then
	; selection exists
	if (ScaleToTimeSelectionBTL ()) then
		; scale to that selection
		Let span = selectionLength
		Let base = selectionStart
	else
		; show this selection,  but scale to the whole project length
		Let showSelectedRegion = 1
	endif
endif ; selection exists
if (ScaleToCustomBTL()) then
	Let base = CustomScalingStartBTL
	Let span = CustomScalingLengthBTL
endif

Let cellCount = BrailleGetCellCount ()

; use current playback position,  or last pressed routing key
if (IsTimelineStatic()) then
	Let sCursor = TimeLineLastPressTimeRPR
else
	Let sCursor = apiGetCurrentPositionMinSec ()
endif
Let R = TimeTomilliSeconds (sCursor) 

; where is the cursor
Let iCursor = interpolateBTL (base, r, span, cellCount)
if (iCursor >= cellCount) then
	Let iCursor = cellCount 
endif

; if the cursor is below the base,  display nothing
if (r < base) then
	return ("                    ")
endif

; compose the output line 
if (showSelectedRegion) then
	; scale to entire project, but show dashes where the selection exists
	Let iSelectionStart = interpolateBTL (0, selectionStart, span, cellCount)
	Let iSelectionEnd   = interpolateBTL (0, selectionEnd, span, cellCount)
	Let iSelectionLength = (iSelectionEnd - iSelectionStart) + 1
	; there are three cases here
	if (iCursor < iSelectionStart) then
		; cursor is left of selection
		Let sLine = StringLeft (spaces80, iCursor) + cs_cursor
		Let sLine = sLine + StringLeft (spaces80, (iSelectionStart - iCursor)-1)
		Let sLine = sLine + StringLeft (dashes80, iSelectionLength)
	elif (iCursor > iSelectionEnd) then
		; cursor is right of the selection
		Let sLine = stringLeft (spaces80, iSelectionStart)
		Let sLine = sLine + stringLeft (dashes80, iSelectionLength)
		Let sLine = sLine + StringLeft (spaces80, (iCursor - iSelectionEnd)-1) + cs_cursor
	else
		; cursor is inside the selection
		Let sLine = StringLeft (spaces80, iSelectionStart)
		Let sLine = sLine + StringLeft (dashes80, (iCursor - ISelectionStart)) + "="
		Let sLine = sLine + StringLeft (dashes80, (iSelectionEnd - iCursor))
	endif
else
	; not showing the selection
	if (ScaleToTimeSelectionBTL ()) then
		; make the background dashes to show we are in the selection.
		Let sLine = StringLeft (dashes80, iCursor) + "="
		; add trailing dashes
		Let sLine = sLine + StringLeft (dashes80, (cellCount - iCursor))
	else
		Let sLine = StringLeft (spaces80, iCursor) + cs_cursor
	endif
endif
return (sLine)
EndFunction


Int Function ToggleBrailleTimeLine (int r)
var
	string sTime,
		int status = 0
	
if  (r == 1) then
	; toggle the timeline mode on or off
	if ((TimeLineModeRPR & 1) == 0) then
		Let TimeLineModeRPR = TimeLineModeRPR | 1
		SayString (msgTimeLineModeActive)
	else
		Let TimeLineModeRPR = (TimeLineModeRPR & 0xffffffe) ; bit off
		SayString (msgNormalModeActive)
	endif
	Let status = 1
elif (r == 2) then
	; change modes track play cursor,  or stay where you are
	if ((TimeLineModeRPR & 2) == 0) then
		Let TimeLineModeRPR = TimeLineModeRPR | 2
		SayString (msgStayWhereYouAre)
	else
		Let TimeLineModeRPR = (TimeLineModeRPR & 0xffffffd) ; bit off
		SayString (msgTrackPlayCursor)
	endif
	Let status = 1
elif (r == 3) then
	; track the project,  the time selection,  or custom marking
	Let scalingBTL = ScalingBTL+1
	if (ScalingBTL > 2) then
		Let ScalingBTL = 0
	endif
	SayString (StringSegment (msgScalingmodesBTL, "|", ScalingBTL+1))
	if (scalingBTL == 1) then
		Let sTime = apiGetSelectionLength (tf_minsec)
		if (sTime == "0:0.000") then
			SayString (msgNoSelection_s)
		endif
	endif
	Let status = 1
elif (r == 4) then
	; mark the selection as a custom scale
	CaptureCustomScalingBTL()
	Let status = 1
endif
return (status)
EndFunction

Function ShowTrackRulerBTL ()
Let TimeLineModeRPR = TimeLineModeRPR | 4
EndFunction


Function HideTrackRulerBTL ()
Let TimeLineModeRPR = TimeLineModeRPR & 0xfffffffb
EndFunction


Int Function ShowingTrackRulerBTL ()
var
	int status = 0
	
if (TimeLineModeRPR & 4) then
	status = 1
endif
return (status)
EndFunction


Function TrackRulerMonitorBTL ()
; scheduled to watch for release of the shift key 
var
	int keyState,
	int toggle

GetKeyState ("shift", keyState, toggle)
if (keyState) then
	; still pressed
	ScheduleFunction ("TrackRulerMonitorBTL", 1)
else
	HideTrackRulerBTL ()
endif
EndFunction




Script BrailleSelectText ()
var
	int r,
	string class
	
Let class = GetWindowClass (hReaperFocus)
if (IsTrackList (class)) then	
	if (ShowingTrackRulerBTL()) then
		Let r = GetLastBrailleRoutingKey ()
		SelectThisTrack (IntToString (r))
	else
		ShowTrackRulerBTL()
		SayString (msgSelectATrack)
		ScheduleFunction ("TrackRulerMonitorBTL", 1)
	endif
endif
EndScript

