;reaper undo History
Import "snow_msaa.jsd"
include "reaper.jsm"

globals
	int rprUndoActive,
	int rprUndoColumn

; the field order here corresponds with logid in undoSpeakField
const
		undoFieldNames = "all|Name|Date|Time",
		undoFieldCount = 4
		
	

Function UndoHistoryFocus (handle hFocus)
; called from focus changed Event
; sets the global flag if we are focused
var
	handle hParent

Let rprUndoActive = 0
if (GetWindowClass (hFocus) == "SysListView32") then
	Let hParent = getParent (hFocus)
	if (GetWindowName (hParent) == "Undo History") then
		Let rprUndoActive = 1
	endif
endif
EndFunction


Int Function UndoIsActive ()
return (rprUndoActive)
EndFunction



Function undoPriorDifferentItem ()
var
	object o,
	int id,
	int maxID,
	string sStartingName,
	string sName
	
let o = ObjGetFocus (id)
Let id = o.accFocus
Let sStartingName = o.accName(id)
Let sName = sStartingName

while (id > 1)
	if (sName != sStartingName) then
		; found a change
		speechOff()
		ObjSetFocus (o, id)
		pause()
		speechOn()
		undoSpeakField(0)
		return
	endif
	Let id = id-1
	Let sName = o.accName(id)
endWhile
SayString (msgNoNewName)
EndFunction


Function undoNextDifferentItem ()
var
	object o,
	int id,
	int maxID,
	string sStartingName,
	string sName
	
let o = ObjGetFocus (id)
Let id = o.accFocus
Let maxID = o.accChildCount
Let sStartingName = o.accName(id)
Let sName = sStartingName

while (id < maxID)
	if (sName != sStartingName) then
		; found a change
		SpeechOff()
		ObjSetFocus (o, id)
		pause()
		speechOn()
		undoSpeakField(0)
		return
	endif
	Let id = id+1
	Let sName = o.accName(id)
endWhile
SayString (msgNoNewName)
EndFunction


Function undoNextField ()
Let rprUndoColumn = rprUndoColumn+1
if (rprUndoColumn > undoFieldCount) then
	Let rprUndoColumn = undoFieldCount
endif
SayString (undoGetFieldName (rprUndoColumn))
undoSpeakField (0)
EndFunction


Function undoPriorField ()
Let rprUndoColumn = rprUndoColumn-1
if (rprUndoColumn<= 0) then
	Let rprUndoColumn = 1
endif
sayString (undoGetFieldName (rprUndoColumn))
undoSpeakField (0)
EndFunction


String Function undoGetFieldName (int iField)
return (StringSegment (undoFieldNames, "|", iField))
EndFunction


Function undoSpeakField (int iField)
; iField==0 from sayWord,  or -1 for an up down arrow key
var
	object o,
	int id,
	string sName,
	string sDescription,
	string sOut

if (iField <= 0) then
	if (iField < 0) then
		; an arrow key
		SpeechOff()
		TypeCurrentScriptKey ()
		pause()
		SpeechOn()
	endif
	Let iField = rprUndoColumn
endif

let o = ObjGetFocus (id)
Let id = o.accFocus

Let sName = o.accName(id)
Let sDescription = o.accdescription(id)

if (iField == 1) then
	; all
	Let sOut = sName + ", " + sDescription
elif (iField == 2) then
	Let sOut = sName
elif (iField == 3) then
		Let sOut = StringSegment (sDescription, " ", 2)
elif (iField == 4) then
		Let sOut = StringSegment (sDescription, " ", 3) +
				StringSegment (sDescription, " ", 4) 		
endif
SayString (sOut) 
EndFunction






